pro shapelets_guess_nmaxbeta, obj, guess,              $
                              TH_MIN_GEOM=th_min_geom, $
                              PRINTIT=printit

;$Id: shapelets_guess_nmaxbeta.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; NAME:
;      SHAPELETS_GUESS_NMAXBETA
;
; PURPOSE:
;      Guess the optimal beta and nmax for an object.
;
; CATEGORY:
;      Shapelets.
;
; INPUTS:
;      obj - IDL "object" structure from shapelets_extract_pstamp.pro.
;
; OPTIONAL INPUTS:
;      th_min_geom - minimum scale for geometrical constraint.
;                    (default: 0.5 pixels)
;
; KEYWORD PARAMETERS:
;      /PRINTIT - if this is set, the guesses are written to stdout.
;
; OUTPUTS:
;      guess - IDL structure containing guessed n_max, beta etc.
;
; MODIFICATION HISTORY:
;      Oct 03 - Beta guess changed from theta_min/max to a calibrated
;               formula based upon SExtractor's FWHM measurement by RM.
;      May 02 - Written by A. Refregier
;-

; guess centroid using sextractor centroid
x0=[obj.xo,obj.yo]

; compute geometrical limits for th_max and th_min
th_max_geom=min([x0(0),obj.im_ran(1)-obj.im_ran(0)-x0(0),$
            x0(1),obj.im_ran(3)-obj.im_ran(2)-x0(1)])
if not keyword_set(th_min_geom) then th_min_geom=1.

; guess th_min and th_max
th_min=th_min_geom(0)
th_max=(2.1*obj.a)<th_max_geom

; compute beta and nmax for this guess
;beta=sqrt(th_min*th_max)
beta=obj.fwhm/2.86 ; Constant determined by investigating results using rule above
nmax=round(th_max/th_min);-1
if (nmax mod 2) then nmax=nmax+1

; print guesses to screen if requested
if keyword_set(printit) then begin
  print,'guess: x0, y0;',x0(0),x0(1)
  print,'guess: beta, nmax:',beta,nmax
  print,'guess: th_min, th_max:',th_min,th_max
  print,'geometry: th_min, th_max:',th_min_geom,th_max_geom
endif

; store in structure
guess={beta:beta,nmax:nmax,x0:x0,th_min:th_min,th_max:th_max,$
  th_min_geom:th_min_geom,th_max_geom:th_max_geom}

end
