pro shapelets_make_chi2_grid,obj,chi2_grid,plotit=plotit,beta_range=beta_range,n_ran=n_ran,$
    printit=printit,n_beta=n_beta,x0=x0

; April 2002 - Modified by AR to use least-square fitting rather than
;              linear decomposition
; August 2000 - Written by A. Refregier
; PURPOSE: compute the chi2 difference between the input and the reconstructed
; image on a grid of values of beta and nmax
; INPUT: obj: object structure from shapelets_extract_pstamp.pro
; OPTIONAL INPUT: plotit: 1: plot chi2 array; 2: also plot reconstructed images
;                 beta_range,n_ran: beta and nmax range (default: educated guess)
;                 n_beta: number of beta values to consider (default=10)
;                 printit: print results
;                 x0: centroid position (default: guess it from image moments)
; OUTPUT: chi2_grid: result structure

; declarations
sf=size(obj.im)
n_f=[sf(1),sf(2)]
n_d=n_f(0)*n_f(1)
if not keyword_set(n_beta) then n_beta=10      ; number of beta values

; compute guess for beta and n_max
shapelets_image_moments,obj.im,mom             ; compute first few moments
th_max=mom.r*1.
th_min=1.
beta_guess=sqrt(th_max*th_min)
n_guess=round(th_max/th_min)
if not keyword_set(x0) then x0=mom.xc
if keyword_set(printit) then print,'initial guess: w, n,x0:',beta_guess,n_guess,x0

; select range of beta and n_max 
if not keyword_set(n_ran) then n_ran=[0,n_guess*2<12]  ; range of n_max values
n_n=n_ran(1)-n_ran(0)+1
n=indgen(n_n)+n_ran(0)
n_cof=(1+n)*(2+n)/2   ; number of coefficients for given n_max
if not keyword_set(beta_range) then beta_range=[beta_guess/2.,beta_guess*2.]
beta=xgen(beta_range(0),beta_range(1),np=n_beta)

; decompose the image for different values of beta and max and compute chi^2
!p.region=0
if keyword_set(plotit) then begin
  if plotit eq 2 then begin
    !p.multi=[0,3,3]
  endif
endif
;plt_image,f
chi2a=fltarr(n_n,n_beta)
chi2_min=1e10
if keyword_set(plotit) then r=1
for i=0,n_beta-1 do begin
  for j=0,n_n-1 do begin
    shapelets_decomp,obj.im,beta(i),n(j),d,r,x0=x0,/over,/ls,/integrate,$
      noise=obj.noise
    chi2a(j,i)=d.chisq(1)
    if keyword_set(plotit) then begin
      if plotit eq 2 then begin 
        if ((i*n_n+j) mod 9 eq 0) then read,test
        print,'beta, n, chi^2:',beta(i), n(j),chi2a(j,i)
        plt_image,r
      endif
    endif
    if chi2a(j,i) lt chi2_min then begin
      chi2_min=chi2a(j,i)
      beta_best_fit=beta(i)
      n_best_fit=n(j)    
    endif
  endfor
endfor

; print best fit values
if keyword_set(printit) then begin
  print,'best fit: beta, n_max:',beta_best_fit,n_best_fit
  print,'theta_min,theta_max:', beta_best_fit/sqrt(n_best_fit+1),beta_best_fit*sqrt(n_best_fit+1)
endif

;; plot chi^2 vs beta and n_max
;!p.multi=0
;plt_image,chi2,frame=[n_ran(0),n_ran(1)+1,beta_range(0),beta_range(1)],/col,$
;  xtitle='n_max',ytitle='beta'
;oplot,[n_guess],[beta_guess],psym=6
;oplot,[n_best_fit],[beta_best_fit],psym=1
;ww=xgen(beta_range(0),beta_range(1))
;oplot,ww^2,ww

; store best fit values in structure
chi2_grid={beta_best_fit:beta_best_fit,nmax_bf:n_best_fit,beta:beta,nmax:n,chi2:chi2a,beta_guess:beta_guess,nmax_g:n_guess}

; plot chi^2 vs beta and n_max
if keyword_set(plotit) then begin
  if plotit eq 2 then read,test
  !p.multi=0
  plt_chi2_grid,chi2_grid
endif

end


