function shapelets_p2c, polar_coeffs, expand=expand

;$Id: shapelets_p2c.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; Richard Massey - February 2002
;
; Converts input POLAR shapelet coefficients to CARTESIAN shapelet coefficients.
; INPUT:    polar_coeffs - just the array of coefs - so it will work for a 
;                          decomp or a catalogue structure
; KEYWORDS: expand - reinserts degenerate coefficients. Use only if they had
;                    been previously removed with shapelets_polar_reduce.pro
;-

; Work out what n_max was and set up variables
polar_coeffs=reform(polar_coeffs)
n_coeffs = n_elements(polar_coeffs)
n_max=fix(0.5*(sqrt(8.*n_coeffs+1.)-1.))-1
shapelets_make_nvec, n_max, n1, n2, n_n

; Re-insert degenerate coefficients
if keyword_set(expand) then polar_coeffs=shapelets_polar_expand(polar_coeffs)

; Create conversion matrix
matrix=complexarr(n_coeffs,n_coeffs)
for i=0,n_coeffs-1 do begin    ; Cartesian
  for j=0,n_coeffs-1 do begin  ; Polar

    ; Get into form of eqn(69) in Paper I
    n_r=n1(j)
    n_l=n2(j)
    n_1=n1(i)
    n_2=n2(i)

    if n_1+n_2 eq n_r+n_l then begin

      factor=complex(0.,0.)
      for n_rd=0,n_r do begin
	   for n_ld=0,n_l do begin
         if n_rd+n_ld eq n_1 then begin
		   factor = factor                $
		    + complex(0.,1.)^(n_ld-n_rd) $
		    * comb(n_r,n_rd)             $
		    * comb(n_l,n_ld)
         endif
	   endfor
	 endfor

	 matrix(j,i) =               $
	     2.^(-0.5*(n_r+n_l))     $
	   * complex(0,1.)^(n_r-n_l)  $
	   * sqrt( factorial(n_1)    $
	        *  factorial(n_2)    $
	        /  factorial(n_r)    $
		   /  factorial(n_l)  ) $
	   * factor


    endif

  endfor
endfor


; Do conversion
cartesian_coeffs=float(invert(matrix)#polar_coeffs)

return,cartesian_coeffs

end

