pro shapelets_extend_nmax,decomp,ninc

;$Id: shapelets_extend_nmax.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; Apr 02 - Written by Richard Massey
;
; Increases the value of n_max in a structure by ninc.
; All the input coefs etc are kept the same, and the new a's are set to zero.
;
; NB: since the n_elements of structure variables can't change, it has to be
;     read out of the structure and a whole new structure created. If new
;     variables or flags are added to the structure in shapelets_decomp, they
;     will also need to be updated here.
;-

; read from original structure
coeffs    = decomp.coeffs
error     = decomp.error
if n_elements(decomp.chisq)     ne 0 then chisq     = decomp.chisq     else chisq=0
if n_elements(decomp.skyfit)    ne 0 then skyfit    = decomp.skyfit    else skyfit=0
if n_elements(decomp.over)      ne 0 then over      = decomp.over      else over=1
if n_elements(decomp.integrate) ne 0 then integrate = decomp.integrate else integrate=1
if n_elements(decomp.name)      ne 0 then name      = decomp.name      else name=""
if n_elements(decomp.type)      ne 0 then type      = decomp.type      else type="cartesian_decomp"
if n_elements(decomp.history)   ne 0 then history   = decomp.history   else history=""
if n_elements(decomp.beta)      ne 0 then beta      = decomp.beta      else beta=0
if n_elements(decomp.moments)   ne 0 then moments   = decomp.moments   else moments=0B
if n_elements(decomp.sex)       ne 0 then sex       = decomp.sex       else sex=0B

; change nmax
n_max_old = decomp.n_max
n_max     = n_max_old+ninc
shapelets_make_nvec,n_max,n1,n2,n_coeffs
while n_elements(coeffs) lt n_coeffs do begin
  coeffs = [coeffs,0.]
   error = [error,0.]
endwhile

; enlarge pixellised region
sizeinc = sqrt((n_max+1)/(n_max_old+1))  ; rather ad hoc rule to guess increased number of pixels needed
n_pixels= decomp.n_pixels + 2*round(sizeinc)*[1,1]
x       = decomp.x + round(sizeinc)*[1.,1.]

; store in a new structure
decomp = {name:name,              $
		type:type, 		    $
		history:history,	    $
		x:x,                    $
		beta:beta,		    $
		n_max:n_max,		    $
		coeffs:coeffs,          $
		n_coeffs:n_coeffs,      $
		n_pixels:n_pixels,      $
		moments:decomp.moments, $
		sex:decomp.sex,	    $
;		w:beta		         $ ; Obsolescent |
;		nmax:n_max,		    $ ;			|
;		x0:x,                   $ ;			V
		n1:n1,			    $
		n2:n2,			    $
;		a:coeffs, 		    $
;		n_a:n_coeffs,  	    $
;		nf:n_pixels,            $
		over:over,		    $
		integrate:integrate,    $
		error:error,  	  	    $
		chisq:chisq, 	  	    $
		skyfit:skyfit}

end
