pro shapelets_rotate, decomp, angle, centre=centre, matrix=matrix

;$Id: shapelets_rotate.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; Richard Massey - February 2002
;
; Rotates an object by converting it`s Cartesian shapelet coefficients to
; polar shapelets, then applying the rotation matrix - which, in this basis,
; is a simple multiplication.
;
; INPUT: decomp: A shapelet decomposition (unaffected except for coefficients
;                and errors on coefficients)
;        angle:  Clockwise, in degrees
;        centre: Centre of rotation [default: shapelet centre]
;                Apply Translation operator before & after?
;-

; Convert to polar shapelet coefficients
if keyword_set(matrix) then begin
  polar_coeffs=matrix#decomp.coeffs
endif else begin
  polar_coeffs=shapelets_c2p(decomp.coeffs,matrix=matrix)
endelse
polar_error=matrix#decomp.error

;; Assemble rotation matrix
;angle=!pi/180.*angle
;for j=0,decomp.n_coeffs-1 do begin
;  n_r=decomp.n1(j)
;  n_l=decomp.n2(j) ; m = n_r-n_l
;  polar_coeffs(j)=polar_coeffs(j)*exp(complex(0,1) * angle * (n_r-n_l) )
;endfor

; Convert coefficients to {mag,phase}
phase    = atan(imaginary(polar_coeffs),double(polar_coeffs))
modulus  = double(abs(polar_coeffs))
ephase   = atan(imaginary(polar_error),double(polar_error))
emodulus = double(abs(polar_error))

; Rotation is just changing the phase of coefficient, modulated by their spin
angle   = !pi/180.*angle
m       = decomp.n1-decomp.n2
phase   = phase+angle*m
ephase  = ephase+angle*m

; Convert back to {real,imaginary}
polar_coeffs     = modulus*complex(cos(phase),sin(phase))
polar_error      = emodulus*complex(cos(ephase),sin(ephase))

; Convert back to Cartesian shapelet coefficients
;decomp.coeffs=shapelets_p2c(polar_coeffs)
matrix=invert(matrix)
decomp.coeffs=matrix#polar_coeffs
decomp.error=matrix#polar_error

end
