pro plt_decomp, decomp, recomp, nran=nran, coef=coef, top=top, ps=ps,$
  noover=noover, cran=cran, title=title, frame=frame, pow=pow, polar=polar, $
  xtitle=xtitle, ytitle=ytitle, scalable=scalable, errors=errors

;$Id: plt_decomp.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; Mar 02 - Polar shapelet options (using plt_polar.pro) added by RM
; Dec 01 - Power spectrum, errors etc. incorporated by R. Massey
; Jul 99 - Written by A. Refregier
;
; PURPOSE: plot the function corresponding to the decomposition
; INPUT: decomp: structure produced by shapelets_decomp
; OPTIONAL INPUT: coef: plot the coefficients array instead of the
;                polar: convert to and plot polar basis functions using plt_polar
;                       reconstructed image.
;                       (1) Magnitude
;                       (2) Relative phase
;                       (3) Composite, half and half
;                 nran: ignore coefficients with n outsdide of nran
;                  top: keep top largest coefficients
;               noover: do not oversample the basis functions
;                  pow: overlay shapelet power spectrum
;               errors: 1 - plot S/N of object or coefficients with /coef
;                         2 - plots absolute errors
;
; OUTPUT: recomp: reconstructed image array
;         plot
;-

; declarations
;csize=.12

; set n range
if not keyword_set(nran) then nran = [0,decomp.n_max]
n_n=nran(1)-nran(0)+1
;n_coeffs=(nran(1)*(nran(1)+1)-nran(0)*(nran(0)+1))/2

; Plot errors on coefficients if requested
if keyword_set(errors) then begin
  if not keyword_set(title) then title=''
  temp_decomp=decomp
  if errors eq 1 then begin
    temp_decomp.coeffs=abs(temp_decomp.coeffs)/temp_decomp.error
    title='S/N on coefficients '+title
  endif else begin
    temp_decomp.coeffs=temp_decomp.error
    title='Errors of coefficients '+title
  endelse
  plt_decomp,temp_decomp, recomp, nran=nran, coef=coef, top=top, ps=ps,$
    noover=noover, cran=cran, title=title, scalable=scalable, polar=polar, frame=frame
  return
endif

; Plot polar coefficients if requested
if keyword_set(polar) then begin
  plt_polar, decomp, nran=nran, coef=coef, top=top, ps=ps, format=polar, $
    cran=cran, title=title, frame=frame, pow=pow, scalable=ps
  return
endif


; open ps file if requested
if keyword_set(ps) then begin
  ops,form=1
  csize=.23
;  loadct,0
;  gamma_ct,.2
endif

; plot reconstructed image
if not keyword_set(coef) then begin
  shapelets_recomp,decomp,recomp,nran=nran,top=top,noover=noover
  plt_image,recomp,/col,scalable=ps,inv=ps,csize=csize,cran=cran,$
    frame=frame,title=title,xtitle=xtitle,ytitle=ytitle

; plot coefficient array instead if requested
endif else begin
  if not keyword_set(nran) then nran = [0,decomp.n_max]
  if keyword_set(top) then begin
    sorted=reverse(sort(abs(decomp.coeffs)))
    a_top=abs(decomp.coeffs(sorted(top-1)))
  endif else begin
    a_top=0.
  endelse
  n_n=nran(1)+1
  aa=fltarr(nran(1)+1,nran(1)+1)
  for i=0,decomp.n_coeffs-1 do begin
    ni=decomp.n1(i)+decomp.n2(i)
    if ni ge nran(0) and ni le nran(1) and abs(decomp.coeffs(i)) ge a_top then $
       aa(decomp.n1(i),decomp.n2(i))=decomp.coeffs(i)
  endfor
  plt_image,aa,/col,frame=[-.5,nran(1)+.5,-.5,nran(1)+.5],scal=ps,cs=csize,inv=ps,$
    xtitle='!6n!i1!n',ytitle='!6n!i2!n',title=title,cran=cran
endelse

if keyword_set(pow) and keyword_set(coef) then begin
  pows=alog10(power_spect(decomp)) ; shapelet spectrum - then stretch to fit
  pows=pows(0:round(.9*decomp.n_max))
  pows=(pows-min(pows))*decomp.n_max/(max(pows)-min(pows))+.5
  oplot,indgen(decomp.n_max+1)+0.5,pows,thick=3
endif

; close ps file if appropriate
if keyword_set(ps) then begin
  cps
endif
end










