pro plt_focus,obj,shapelets_focus,decomp,chi2_grid=chi2_grid,centroid=centroid

;$Id: plt_shapelets_focus.pro, v1.0$
;
; Copyright  2004 Richard Massey and Alexandre Refregier.
;
; This file is a part of the Shapelets analysis code.
; www.ast.cam.ac.uk/~rjm/shapelets/
;
; The Shapelets code is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License as published
; by the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; The Shapelets code is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Shapelets code; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
;
;+
; May 2002   - Minor style adjustments by R. Massey
; April 2002 - Written by A. Refregier
; PURPOSE: plot the results of the shapelets_focussing of shapelets on an object.
; The history of the search of optimal shapelet parameters is also shown.
; INPUT: obj: object structure from shapelets_image2obj.pro
;        shapelets_focus: shapelets_focus structure from mk_shapelets_focus.pro    
;        decomp: decomposition structure from mk_shapelets_focus.pro  
; OPTIONAL INPUT: chi2_grid: plot the Chi^2(beta,nmax) grid array if provided.
;                   if this keyword is set to a scalar the array is
;                   calculated with ranges around the optimal values
;                 centroid: plot centroid search history
; OUTPUT: plots
;-

; declarations
th_max=shapelets_focus.beta*sqrt(shapelets_focus.nmax+1.)
th_min=shapelets_focus.beta/sqrt(shapelets_focus.nmax+1.)
th_maxmax=abs(min([shapelets_focus.x0(0),obj.im_ran(1)-obj.im_ran(0)-shapelets_focus.x0(0),$ 
            shapelets_focus.x0(1),obj.im_ran(3)-obj.im_ran(2)-shapelets_focus.x0(1)]))
                  ; imposed by image boundaries

; plot reconstructed object
r=1.
shapelets_decomp,obj.image,shapelets_focus.beta,shapelets_focus.nmax,decomp,r,$
    x0=shapelets_focus.x0,/ls,/over,noise=obj.noise,/integrate
window,2,xsize=500,ysize=555,xpos=130,ypos=350
c_ran=[min(obj.image),max(obj.image)]
plt_image,r,/fr,/col,cran=c_ran,$
  title='(Deconvolved) object reconstructed to n!dmax!n='+strtrim(shapelets_focus.nmax,1)
oplot,[shapelets_focus.x0(0)],[shapelets_focus.x0(1)],psym=1
plt_ellipse,shapelets_focus.x0(0),shapelets_focus.x0(1),shapelets_focus.beta,shapelets_focus.beta,0.
plt_ellipse,shapelets_focus.x0(0),shapelets_focus.x0(1),th_min,th_min,0.,lines=2
plt_ellipse,shapelets_focus.x0(0),shapelets_focus.x0(1),th_max,th_max,0.,lines=2


; plot search history on beta vs nmax plane
window,3,xsize=500,ysize=300,xpos=640,ypos=0
!p.region=0
if keyword_set(chi2_grid) then begin
  beta_ran=shapelets_focus.beta*[.5,1.5] & nmax_ran=[2,(shapelets_focus.nmax+2)<15]
  s=size(chi2_grid)
  if s(0) eq 0 then begin
    print,'chi^2 grid: nmax_min, nmax_max:',nmax_ran(0),nmax_ran(1)
    print,'chi^2 grid: beta_min, beta_max:',beta_ran(0),beta_ran(1)
    shapelets_make_chi2_grid,obj,chi2_grid,beta_ran=beta_ran,n_ran=nmax_ran,x0=shapelets_focus.x0
  endif
  plt_chi2_grid,chi2_grid;,title='Search history',xtit='n!dmax!n',ytit='beta'
  contour,chi2_grid.chi2,chi2_grid.nmax,chi2_grid.beta,$
    level=shapelets_focus.chi20+shapelets_focus.tol_chi2*[-2.,-1.,0.,1.,2.],$
    /over,c_lines=[1,2,0,2,1]
endif else begin
;beta_ran=[shapelets_focus.beta*.5,shapelets_focus.beta*1.5]
beta_ran=[0.,th_maxmax]
nmax_ran=[0,max(shapelets_focus.nmax_h)+2]
plot,shapelets_focus.nmax_h,shapelets_focus.beta_h,/nodata,yran=beta_ran,xran=nmax_ran,$
    xtitle='!6n!imax!n',ytitle='!7b!6'
endelse
oplot,shapelets_focus.nmax_h,shapelets_focus.beta_h
oplot,shapelets_focus.nmax_h,shapelets_focus.beta_h,psym=1
oplot,[shapelets_focus.nmax_guess],[shapelets_focus.beta_guess],psym=7
oplot,[shapelets_focus.nmax],[shapelets_focus.beta],psym=6

; plot geometrical constraints
nnmax=xgen(0.,nmax_ran(1)+4)
oplot,nnmax,shapelets_focus.th_min*sqrt(nnmax+1.),lines=3
oplot,nnmax,th_maxmax/sqrt(nnmax+1.),lines=3

; plot history of search of centroid if requested
if keyword_set(centroid) then begin
  window,4,xsize=500,ysize=500,xpos=130,ypos=0
  !p.region=0
  plot,[reform(shapelets_focus.x0_h(0,*)),shapelets_focus.x0(0),shapelets_focus.x0_guess(0)],$
    [reform(shapelets_focus.x0_h(1,*)),shapelets_focus.x0(1),shapelets_focus.x0_guess(1)],$
    /nodata,/ynozero,xtitle='x',ytitle='y',title='Centroid search'
  oplot,shapelets_focus.x0_h(0,*),shapelets_focus.x0_h(1,*)
  oplot,[shapelets_focus.x0_guess(0)],[shapelets_focus.x0_guess(1)],psym=7
  oplot,[shapelets_focus.x0(0)],[shapelets_focus.x0(1)],psym=6
endif

; plot object
window,1,xsize=500,ysize=555,xpos=640,ypos=350
plt_pstamp,obj,/oframe,cran=c_ran,title='Original object'
oplot,shapelets_focus.x0_h(0,*),shapelets_focus.x0_h(1,*)
oplot,[shapelets_focus.x0,shapelets_focus.x0],[0,300],linestyle=2
oplot,[0,300],[shapelets_focus.x0,shapelets_focus.x0],linestyle=2

end


